@@ -36,7 +36,7 @@ const DashItemContainer = new Lang.Class({
                            Lang.bind(this, this._allocate));
         this.actor._delegate = this;
 
-        this._label = null;
+        this.label = null;
 
         this.child = null;
         this._childScale = 1;
@@ -91,32 +91,32 @@ const DashItemContainer = new Lang.Class({
     },
 
     showLabel: function() {
-        if (this._label == null)
+        if (this.label == null)
             return;
 
-        this._label.opacity = 0;
-        this._label.show();
+        this.label.opacity = 0;
+        this.label.show();
 
         let [stageX, stageY] = this.actor.get_transformed_position();
 
         let itemHeight = this.actor.allocation.y2 - this.actor.allocation.y1;
 
-        let labelHeight = this._label.get_height();
+        let labelHeight = this.label.get_height();
         let yOffset = Math.floor((itemHeight - labelHeight) / 2)
 
         let y = stageY + yOffset;
 
-        let node = this._label.get_theme_node();
+        let node = this.label.get_theme_node();
         let xOffset = node.get_length('-x-offset');
 
         let x;
-        if (St.Widget.get_default_direction () == St.TextDirection.RTL)
-            x = stageX - this._label.get_width() - xOffset;
+        if (Clutter.get_default_text_direction() == Clutter.TextDirection.RTL)
+            x = stageX - this.label.get_width() - xOffset;
         else
             x = stageX + this.actor.get_width() + xOffset;
 
-        this._label.set_position(x, y);
-        Tweener.addTween(this._label,
+        this.label.set_position(x, y);
+        Tweener.addTween(this.label,
                          { opacity: 255,
                            time: DASH_ITEM_LABEL_SHOW_TIME,
                            transition: 'easeOutQuad',
@@ -124,22 +124,22 @@ const DashItemContainer = new Lang.Class({
     },
 
     setLabelText: function(text) {
-        if (this._label == null)
-            this._label = new St.Label({ style_class: 'dash-label'});
+        if (this.label == null)
+            this.label = new St.Label({ style_class: 'dash-label'});
 
-        this._label.set_text(text);
-        Main.layoutManager.addChrome(this._label);
-        this._label.hide();
+        this.label.set_text(text);
+        Main.layoutManager.addChrome(this.label);
+        this.label.hide();
     },
 
     hideLabel: function () {
-        this._label.opacity = 255;
-        Tweener.addTween(this._label,
+        this.label.opacity = 255;
+        Tweener.addTween(this.label,
                          { opacity: 0,
                            time: DASH_ITEM_LABEL_HIDE_TIME,
                            transition: 'easeOutQuad',
                            onComplete: Lang.bind(this, function() {
-                               this._label.hide();
+                               this.label.hide();
                            })
                          });
     },
@@ -387,6 +387,7 @@ const Dash = new Lang.Class({
         let srcIsFavorite = (id in favorites);
 
         if (srcIsFavorite &&
+            app.get_state() != Shell.AppState.RUNNING &&
             dragEvent.source.actor &&
             this.actor.contains (dragEvent.source.actor) &&
             this._favRemoveTarget == null) {
@@ -439,6 +440,9 @@ const Dash = new Lang.Class({
         item.setChild(display.actor);
 
         item.setLabelText(app.get_name());
+        // Override default AppWellIcon label_actor
+        display.actor.label_actor = item.label;
+
 
         display.icon.setIconSize(this.iconSize);
         display.actor.connect('notify::hover',
@@ -743,20 +747,10 @@ const Dash = new Lang.Class({
             numChildren--;
         }
 
-        let pos = Math.round(y * numChildren / boxHeight);
+        let pos = Math.floor(y * numChildren / boxHeight);
 
-        if (pos != this._dragPlaceholderPos && pos <= numFavorites) {
-            if (this._animatingPlaceholdersCount > 0) {
-                let appChildren = children.filter(function(actor) {
-                    return actor._delegate &&
-                           actor._delegate.child &&
-                           actor._delegate.child._delegate &&
-                           actor._delegate.child._delegate.app;
-                });
-                this._dragPlaceholderPos = children.indexOf(appChildren[pos]);
-            } else {
-                this._dragPlaceholderPos = pos;
-            }
+        if (pos != this._dragPlaceholderPos && pos <= numFavorites && this._animatingPlaceholdersCount == 0) {
+            this._dragPlaceholderPos = pos;
 
             // Don't allow positioning before or after self
             if (favPos != -1 && (pos == favPos || pos == favPos + 1)) {
@@ -793,6 +787,14 @@ const Dash = new Lang.Class({
                 this._dragPlaceholder.animateIn();
         }
 
+        // Remove the drag placeholder if we are not in the
+        // "favorites zone"
+        if (pos > numFavorites && this._dragPlaceholder) {
+            this._clearDragPlaceholder();
+        }
+        if (!this._dragPlaceholder)
+            return DND.DragMotionResult.NO_DROP;
+
         let srcIsFavorite = (favPos != -1);
 
         if (srcIsFavorite)
@@ -835,6 +837,11 @@ const Dash = new Lang.Class({
                 favPos++;
         }
 
+        // No drag placeholder means we don't wan't to favorite the app
+        // and we are dragging it to its original position
+        if (!this._dragPlaceholder)
+            return true;
+
         Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this,
             function () {
                 let appFavorites = AppFavorites.getAppFavorites();
