@@ -15,6 +15,7 @@ const AutorunManager = imports.ui.autorunManager;
 const CtrlAltTab = imports.ui.ctrlAltTab;
 const EndSessionDialog = imports.ui.endSessionDialog;
 const PolkitAuthenticationAgent = imports.ui.polkitAuthenticationAgent;
+const KeyringPrompt = imports.ui.keyringPrompt;
 const Environment = imports.ui.environment;
 const ExtensionSystem = imports.ui.extensionSystem;
 const Keyboard = imports.ui.keyboard;
@@ -37,6 +38,7 @@ const XdndHandler = imports.ui.xdndHandler;
 const StatusIconDispatcher = imports.ui.statusIconDispatcher;
 const Util = imports.misc.util;
 
+const OVERRIDES_SCHEMA = 'org.gnome.shell.overrides';
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
 DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
 
@@ -70,6 +72,7 @@ let _startDate;
 let _defaultCssStylesheet = null;
 let _cssStylesheet = null;
 let _gdmCssStylesheet = null;
+let _overridesSettings = null;
 
 let background = null;
 
@@ -110,7 +113,9 @@ function _initRecorder() {
         } else {
             // read the parameters from GSettings always in case they have changed
             recorder.set_framerate(recorderSettings.get_int('framerate'));
-            recorder.set_filename('shell-%d%u-%c.' + recorderSettings.get_string('file-extension'));
+            /* Translators: this is a filename used for screencast recording */
+            // xgettext:no-c-format
+            recorder.set_filename(_("Screencast from %d %t") + '.' + recorderSettings.get_string('file-extension'));
             let pipeline = recorderSettings.get_string('pipeline');
 
             if (!pipeline.match(/^\s*$/))
@@ -166,9 +171,11 @@ function start() {
     // and recalculate application associations, so to avoid
     // races for now we initialize it here.  It's better to
     // be predictable anyways.
-    Shell.WindowTracker.get_default();
+    let tracker = Shell.WindowTracker.get_default();
     Shell.AppUsage.get_default();
 
+    tracker.connect('startup-sequence-changed', _queueCheckWorkspaces);
+
     // The stage is always covered so Clutter doesn't need to clear it; however
     // the color is used as the default contents for the Mutter root background
     // actor so set it anyways.
@@ -230,6 +237,9 @@ function start() {
     // Attempt to become a PolicyKit authentication agent
     PolkitAuthenticationAgent.init()
 
+    // Become a prompter for gnome keyring
+    KeyringPrompt.init();
+
     _startDate = new Date();
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
@@ -244,6 +254,9 @@ function start() {
         Scripting.runPerfScript(module, perfOutput);
     }
 
+    _overridesSettings = new Gio.Settings({ schema: OVERRIDES_SCHEMA });
+    _overridesSettings.connect('changed::dynamic-workspaces', _queueCheckWorkspaces);
+
     global.screen.connect('notify::n-workspaces', _nWorkspacesChanged);
 
     global.screen.connect('window-entered-monitor', _windowEnteredMonitor);
@@ -268,17 +281,30 @@ function _checkWorkspaces() {
     let i;
     let emptyWorkspaces = [];
 
+    if (!Meta.prefs_get_dynamic_workspaces()) {
+        _checkWorkspacesId = 0;
+        return false;
+    }
+
     for (i = 0; i < _workspaces.length; i++) {
         let lastRemoved = _workspaces[i]._lastRemovedWindow;
-        if (lastRemoved &&
-            (lastRemoved.get_window_type() == Meta.WindowType.SPLASHSCREEN ||
-             lastRemoved.get_window_type() == Meta.WindowType.DIALOG ||
-             lastRemoved.get_window_type() == Meta.WindowType.MODAL_DIALOG))
+        if ((lastRemoved &&
+             (lastRemoved.get_window_type() == Meta.WindowType.SPLASHSCREEN ||
+              lastRemoved.get_window_type() == Meta.WindowType.DIALOG ||
+              lastRemoved.get_window_type() == Meta.WindowType.MODAL_DIALOG)) ||
+            _workspaces[i]._keepAliveId)
                 emptyWorkspaces[i] = false;
         else
             emptyWorkspaces[i] = true;
     }
 
+    let sequences = Shell.WindowTracker.get_default().get_startup_sequences();
+    for (i = 0; i < sequences.length; i++) {
+        let index = sequences[i].get_workspace();
+        if (index >= 0 && index <= global.screen.n_workspaces)
+            emptyWorkspaces[index] = false;
+    }
+
     let windows = global.get_window_actors();
     for (i = 0; i < windows.length; i++) {
         let win = windows[i];
@@ -326,6 +352,17 @@ function _checkWorkspaces() {
     return false;
 }
 
+function keepWorkspaceAlive(workspace, duration) {
+    if (workspace._keepAliveId)
+        Mainloop.source_remove(workspace._keepAliveId);
+
+    workspace._keepAliveId = Mainloop.timeout_add(duration, function() {
+        workspace._keepAliveId = 0;
+        _queueCheckWorkspaces();
+        return false;
+    });
+}
+
 function _windowRemoved(workspace, window) {
     workspace._lastRemovedWindow = window;
     _queueCheckWorkspaces();
@@ -570,7 +607,8 @@ function _globalKeyPressHandler(actor, event) {
 
     let symbol = event.get_key_symbol();
     let keyCode = event.get_key_code();
-    let modifierState = Shell.get_event_state(event);
+    let ignoredModifiers = global.display.get_ignored_modifier_mask();
+    let modifierState = event.get_state() & ~ignoredModifiers;
 
     // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
     let action = global.display.get_keybinding_action(keyCode, modifierState);
