@@ -487,7 +487,7 @@ const NotificationDaemon = new Lang.Class({
     },
 
     _onTrayIconRemoved: function(o, icon) {
-        let source = this._lookupSource(icon.pid, null, true);
+        let source = this._lookupSource(null, icon.pid, true);
         if (source)
             source.destroy();
     }
