@@ -6,10 +6,12 @@ const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
+const Meta = imports.gi.Meta;
 const Pango = imports.gi.Pango;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
 const Signals = imports.signals;
+const Atk = imports.gi.Atk;
 
 const Config = imports.misc.config;
 const CtrlAltTab = imports.ui.ctrlAltTab;
@@ -249,6 +251,8 @@ const AppMenuButton = new Lang.Class({
     _init: function(menuManager) {
         this.parent(0.0, null, true);
 
+        this.actor.accessible_role = Atk.Role.MENU;
+
         this._startingApps = [];
 
         this._menuManager = menuManager;
@@ -299,7 +303,7 @@ const AppMenuButton = new Lang.Class({
 
         let tracker = Shell.WindowTracker.get_default();
         let appSys = Shell.AppSystem.get_default();
-        tracker.connect('notify::focus-app', Lang.bind(this, this._sync));
+        tracker.connect('notify::focus-app', Lang.bind(this, this._focusAppChanged));
         appSys.connect('app-state-changed', Lang.bind(this, this._onAppStateChanged));
 
         global.window_manager.connect('switch-workspace', Lang.bind(this, this._sync));
@@ -313,11 +317,12 @@ const AppMenuButton = new Lang.Class({
 
         this._visible = true;
         this.actor.show();
-        this.actor.reactive = true;
 
         if (!this._targetIsCurrent)
             return;
 
+        this.actor.reactive = true;
+
         Tweener.removeTweens(this.actor);
         Tweener.addTween(this.actor,
                          { opacity: 255,
@@ -412,12 +417,12 @@ const AppMenuButton = new Lang.Class({
 
         let [minWidth, minHeight, naturalWidth, naturalHeight] = this._iconBox.get_preferred_size();
 
-        let direction = this.actor.get_direction();
+        let direction = this.actor.get_text_direction();
 
         let yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
         childBox.y1 = yPadding;
         childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
-        if (direction == St.TextDirection.LTR) {
+        if (direction == Clutter.TextDirection.LTR) {
             childBox.x1 = 0;
             childBox.x2 = childBox.x1 + Math.min(naturalWidth, allocWidth);
         } else {
@@ -434,7 +439,7 @@ const AppMenuButton = new Lang.Class({
         childBox.y1 = yPadding;
         childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
 
-        if (direction == St.TextDirection.LTR) {
+        if (direction == Clutter.TextDirection.LTR) {
             childBox.x1 = Math.floor(iconWidth / 2);
             childBox.x2 = Math.min(childBox.x1 + naturalWidth, allocWidth);
         } else {
@@ -443,7 +448,7 @@ const AppMenuButton = new Lang.Class({
         }
         this._label.actor.allocate(childBox, flags);
 
-        if (direction == St.TextDirection.LTR) {
+        if (direction == Clutter.TextDirection.LTR) {
             childBox.x1 = Math.floor(iconWidth / 2) + this._label.actor.width;
             childBox.x2 = childBox.x1 + this._spinner.actor.width;
             childBox.y1 = box.y1;
@@ -474,16 +479,9 @@ const AppMenuButton = new Lang.Class({
         this._sync();
     },
 
-    _sync: function() {
+    _focusAppChanged: function() {
         let tracker = Shell.WindowTracker.get_default();
-        let lastStartedApp = null;
-        let workspace = global.screen.get_active_workspace();
-        for (let i = 0; i < this._startingApps.length; i++)
-            if (this._startingApps[i].is_on_workspace(workspace))
-                lastStartedApp = this._startingApps[i];
-
         let focusedApp = tracker.focus_app;
-
         if (!focusedApp) {
             // If the app has just lost focus to the panel, pretend
             // nothing happened; otherwise you can't keynav to the
@@ -491,6 +489,17 @@ const AppMenuButton = new Lang.Class({
             if (global.stage_input_mode == Shell.StageInputMode.FOCUSED)
                 return;
         }
+        this._sync();
+    },
+
+    _sync: function() {
+        let tracker = Shell.WindowTracker.get_default();
+        let focusedApp = tracker.focus_app;
+        let lastStartedApp = null;
+        let workspace = global.screen.get_active_workspace();
+        for (let i = 0; i < this._startingApps.length; i++)
+            if (this._startingApps[i].is_on_workspace(workspace))
+                lastStartedApp = this._startingApps[i];
 
         let targetApp = focusedApp != null ? focusedApp : lastStartedApp;
 
@@ -600,6 +609,7 @@ const ActivitiesButton = new Lang.Class({
 
     _init: function() {
         this.parent(0.0);
+        this.actor.accessible_role = Atk.Role.TOGGLE_BUTTON;
 
         let container = new Shell.GenericContainer();
         container.connect('get-preferred-width', Lang.bind(this, this._containerGetPreferredWidth));
@@ -613,6 +623,8 @@ const ActivitiesButton = new Lang.Class({
         this._label = new St.Label({ text: _("Activities") });
         container.add_actor(this._label);
 
+        this.actor.label_actor = this._label;
+
         this._hotCorner = new Layout.HotCorner();
         container.add_actor(this._hotCorner.actor);
 
@@ -628,10 +640,12 @@ const ActivitiesButton = new Lang.Class({
         Main.overview.connect('showing', Lang.bind(this, function() {
             this.actor.add_style_pseudo_class('overview');
             this._escapeMenuGrab();
+            this.actor.add_accessible_state (Atk.StateType.CHECKED);
         }));
         Main.overview.connect('hiding', Lang.bind(this, function() {
             this.actor.remove_style_pseudo_class('overview');
             this._escapeMenuGrab();
+            this.actor.remove_accessible_state (Atk.StateType.CHECKED);
         }));
 
         this._xdndTimeOut = 0;
@@ -653,7 +667,7 @@ const ActivitiesButton = new Lang.Class({
         let primary = Main.layoutManager.primaryMonitor;
         let hotBox = new Clutter.ActorBox();
         let ok, x, y;
-        if (actor.get_direction() == St.TextDirection.LTR) {
+        if (actor.get_text_direction() == Clutter.TextDirection.LTR) {
             [ok, x, y] = actor.transform_stage_point(primary.x, primary.y)
         } else {
             [ok, x, y] = actor.transform_stage_point(primary.x + primary.width, primary.y);
@@ -807,7 +821,7 @@ const PanelCorner = new Lang.Class({
 
         let rtlAwareContainer = this._box instanceof St.BoxLayout;
         if (rtlAwareContainer &&
-            this._box.get_direction() == St.TextDirection.RTL) {
+            this._box.get_text_direction() == Clutter.TextDirection.RTL) {
             if (this._side == St.Side.LEFT)
                 side = St.Side.RIGHT;
             else if (this._side == St.Side.RIGHT)
@@ -928,14 +942,14 @@ const Panel = new Lang.Class({
         this._rightBox = new St.BoxLayout({ name: 'panelRight' });
         this.actor.add_actor(this._rightBox);
 
-        if (this.actor.get_direction() == St.TextDirection.RTL)
+        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL)
             this._leftCorner = new PanelCorner(this._rightBox, St.Side.LEFT);
         else
             this._leftCorner = new PanelCorner(this._leftBox, St.Side.LEFT);
 
         this.actor.add_actor(this._leftCorner.actor);
 
-        if (this.actor.get_direction() == St.TextDirection.RTL)
+        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL)
             this._rightCorner = new PanelCorner(this._leftBox, St.Side.RIGHT);
         else
             this._rightCorner = new PanelCorner(this._rightBox, St.Side.RIGHT);
@@ -944,6 +958,7 @@ const Panel = new Lang.Class({
         this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
         this.actor.connect('allocate', Lang.bind(this, this._allocate));
+        this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
 
         /* Button on the left side of the panel. */
         if (global.session_type == Shell.SessionType.USER) {
@@ -1011,7 +1026,7 @@ const Panel = new Lang.Class({
 
         childBox.y1 = 0;
         childBox.y2 = allocHeight;
-        if (this.actor.get_direction() == St.TextDirection.RTL) {
+        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL) {
             childBox.x1 = allocWidth - Math.min(Math.floor(sideWidth),
                                                 leftNaturalWidth);
             childBox.x2 = allocWidth;
@@ -1030,7 +1045,7 @@ const Panel = new Lang.Class({
 
         childBox.y1 = 0;
         childBox.y2 = allocHeight;
-        if (this.actor.get_direction() == St.TextDirection.RTL) {
+        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL) {
             childBox.x1 = 0;
             childBox.x2 = Math.min(Math.floor(sideWidth),
                                    rightNaturalWidth);
@@ -1058,6 +1073,45 @@ const Panel = new Lang.Class({
         this._rightCorner.actor.allocate(childBox, flags);
     },
 
+    _onButtonPress: function(actor, event) {
+        if (event.get_source() != actor)
+            return false;
+
+        let button = event.get_button();
+        if (button != 1)
+            return false;
+
+        let focusWindow = global.display.focus_window;
+        if (!focusWindow)
+            return false;
+
+        let dragWindow = focusWindow.is_attached_dialog() ? focusWindow.get_transient_for()
+                                                          : focusWindow;
+        if (!dragWindow)
+            return false;
+
+        let rect = dragWindow.get_outer_rect();
+        let [stageX, stageY] = event.get_coords();
+
+        let allowDrag = dragWindow.maximized_vertically &&
+                        stageX > rect.x && stageX < rect.x + rect.width;
+
+        if (!allowDrag)
+            return false;
+
+        global.display.begin_grab_op(global.screen,
+                                     dragWindow,
+                                     Meta.GrabOp.MOVING,
+                                     false, /* pointer grab */
+                                     true, /* frame action */
+                                     button,
+                                     event.get_state(),
+                                     event.get_time(),
+                                     stageX, stageY);
+
+        return true;
+    },
+
     startStatusArea: function() {
         for (let i = 0; i < this._status_area_order.length; i++) {
             let role = this._status_area_order[i];
