@@ -39,15 +39,23 @@ const GnomeShellIface = <interface name="org.gnome.Shell">
 </method>
 <method name="ScreenshotWindow">
     <arg type="b" direction="in" name="include_frame"/>
+    <arg type="b" direction="in" name="include_cursor"/>
     <arg type="b" direction="in" name="flash"/>
     <arg type="s" direction="in" name="filename"/>
     <arg type="b" direction="out" name="success"/>
 </method>
 <method name="Screenshot">
+    <arg type="b" direction="in" name="include_cursor"/>
     <arg type="b" direction="in" name="flash"/>
     <arg type="s" direction="in" name="filename"/>
     <arg type="b" direction="out" name="success"/>
 </method>
+<method name="FlashArea">
+    <arg type="i" direction="in" name="x"/>
+    <arg type="i" direction="in" name="y"/>
+    <arg type="i" direction="in" name="width"/>
+    <arg type="i" direction="in" name="height"/>
+</method>
 <method name="EnableExtension">
     <arg type="s" direction="in" name="uuid"/>
 </method>
@@ -134,6 +142,7 @@ const GnomeShell = new Lang.Class({
      * @y: The Y coordinate of the area
      * @width: The width of the area
      * @height: The height of the area
+     * @flash: Whether to flash the area or not
      * @filename: The filename for the screenshot
      *
      * Takes a screenshot of the passed in area and saves it
@@ -143,14 +152,17 @@ const GnomeShell = new Lang.Class({
      */
     ScreenshotAreaAsync : function (params, invocation) {
         let [x, y, width, height, flash, filename, callback] = params;
-        global.screenshot_area (x, y, width, height, filename, 
-                                Lang.bind(this, this._onScreenshotComplete, 
+        let screenshot = new Shell.Screenshot();
+        screenshot.screenshot_area (x, y, width, height, filename,
+                                Lang.bind(this, this._onScreenshotComplete,
                                           flash, invocation));
     },
 
     /**
      * ScreenshotWindow:
      * @include_frame: Whether to include the frame or not
+     * @include_cursor: Whether to include the cursor image or not
+     * @flash: Whether to flash the window area or not
      * @filename: The filename for the screenshot
      *
      * Takes a screenshot of the focused window (optionally omitting the frame)
@@ -159,15 +171,18 @@ const GnomeShell = new Lang.Class({
      *
      */
     ScreenshotWindowAsync : function (params, invocation) {
-        let [include_frame, flash, filename] = params;
-        global.screenshot_window (include_frame, filename, 
-                                  Lang.bind(this, this._onScreenshotComplete, 
+        let [include_frame, include_cursor, flash, filename] = params;
+        let screenshot = new Shell.Screenshot();
+        screenshot.screenshot_window (include_frame, include_cursor, filename,
+                                  Lang.bind(this, this._onScreenshotComplete,
                                             flash, invocation));
     },
 
     /**
      * Screenshot:
      * @filename: The filename for the screenshot
+     * @include_cursor: Whether to include the cursor image or not
+     * @flash: Whether to flash the screen or not
      *
      * Takes a screenshot of the whole screen and saves it
      * in @filename as png image, it returns a boolean
@@ -175,12 +190,18 @@ const GnomeShell = new Lang.Class({
      *
      */
     ScreenshotAsync : function (params, invocation) {
-        let [flash, filename] = params;
-        global.screenshot(filename, 
-                          Lang.bind(this, this._onScreenshotComplete, 
+        let [include_cursor, flash, filename] = params;
+        let screenshot = new Shell.Screenshot();
+        screenshot.screenshot(include_cursor, filename,
+                          Lang.bind(this, this._onScreenshotComplete,
                                     flash, invocation));
     },
 
+    FlashArea: function(x, y, width, height) {
+        let flashspot = new Flashspot.Flashspot({ x : x, y : y, width: width, height: height});
+        flashspot.fire();
+    },
+
     ListExtensions: function() {
         let out = {};
         for (let uuid in ExtensionUtils.extensions) {
