@@ -163,6 +163,8 @@ shell_prefs_init (void)
 {
   meta_prefs_override_preference_schema ("attach-modal-dialogs",
                                          OVERRIDES_SCHEMA);
+  meta_prefs_override_preference_schema ("dynamic-workspaces",
+                                         OVERRIDES_SCHEMA);
   meta_prefs_override_preference_schema ("workspaces-only-on-primary",
                                          OVERRIDES_SCHEMA);
   meta_prefs_override_preference_schema ("button-layout",
@@ -217,15 +219,6 @@ shell_perf_log_init (void)
                                           NULL, NULL);
 }
 
-static void
-muted_log_handler (const char     *log_domain,
-                   GLogLevelFlags  log_level,
-                   const char     *message,
-                   gpointer        data)
-{
-  /* Intentionally empty to discard message */
-}
-
 static void
 default_log_handler (const char     *log_domain,
                      GLogLevelFlags  log_level,
@@ -321,18 +314,6 @@ main (int argc, char **argv)
   g_irepository_prepend_search_path (BLUETOOTH_DIR);
 #endif
 
-  /* Disable debug spew from various libraries */
-  g_log_set_handler ("Gvc", G_LOG_LEVEL_DEBUG,
-                     muted_log_handler, NULL);
-  g_log_set_handler ("AccountsService", G_LOG_LEVEL_DEBUG,
-                     muted_log_handler, NULL);
-  g_log_set_handler ("Bluetooth", G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE,
-                     muted_log_handler, NULL);
-  g_log_set_handler ("tp-glib/proxy", G_LOG_LEVEL_DEBUG,
-                     muted_log_handler, NULL);
-  g_log_set_handler ("GnomeShellBrowserPlugin", G_LOG_LEVEL_DEBUG,
-                     muted_log_handler, NULL);
-
   /* Turn on telepathy-glib debugging but filter it out in
    * default_log_handler. This handler also exposes all the logs over D-Bus
    * using TpDebugSender. */
